/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
 * pdn.c                                                     *
 * part of checkers project, contains functions to keep      *
 * history of moves, to save and load games. uses standard   *
 * format PDN (Portable draughts notation)                   *
 * Miroslav Tynovsky 07/2004                                 *
\* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#include <stdlib.h>
#include <assert.h>
#include <stdio.h>
#include "rules_int.h"
#include "pdn.h"

Moves history;

int coords2no(Sq_coords coords);
Sq_coords no2coords(int no);

void insert_move(Move move)
{
	if(history.allocated==0){
		history.allocated= 4;
		history.list= (Move *)malloc(history.allocated * sizeof(Move));
	}
	if(history.allocated==history.count){
		history.allocated*= 2;
		history.list= realloc(history.list, history.allocated * sizeof(Move));
	}
	history.list[history.count]= move;
	history.count++;
}

void history2pdn(char *filename)
{
	int i, j;
	FILE *f;

	f= fopen(filename, "w");
	for(i=0; i<history.count; i++){
		if(i%2==0) fprintf(f, "%d. ", i/2+1);
		switch(history.list[i].move_type){
		case PAWN_STEP:
		case KING_STEP:
		case PROMOTION:
			fprintf(f, "%d-%d ", 
				    coords2no(history.list[i].start), 
			        coords2no(history.list[i].destination));
			break;
		case PAWN_JUMP:
		case KING_JUMP:
		case JUMP_PROMOTION:
			fprintf(f, "%d", coords2no(history.list[i].start));
			for(j=0; j<history.list[i].jump_length-1; j++)
				fprintf(f, "x%d", coords2no(history.list[i].middle[j]));
			fprintf(f, "x%d ", coords2no(history.list[i].destination));
			break;
		default:
			fprintf(f, "%d-%d (movetype %d)", 
				    coords2no(history.list[i].start), 
			        coords2no(history.list[i].destination),
			        history.list[i].move_type);
			break;
		}
		if(i%10==9)
			fprintf(f, "\n");
	}
	fprintf(f, "\n");
	fclose(f);
}

void pdn2history(char *filename)
{
	int i, j, k, start, dest;
	char ch;
	int middle[11];
	FILE *f, *fw;
	Move move;
	Moves poss_moves;
	Position pos;

	initialize_board(&pos);
	history.count= 0;
	if(history.allocated > 0){
		history.allocated= 0;
		free(history.list);
	}

	f= fopen(filename, "r");
	for(i=0; fscanf(f, "%d. ", &start)!=EOF; i++){ 
		//uvnitr for se prirazuje do start, jen aby se nekam priradilo. 
		//hodnota se hned zahodi, je to jen poradove cislo tahu
		fscanf(f, "%d", &start);
		for(j=0; ((ch= getc(f))!=' ') && (ch != EOF); j++)
			fscanf(f, "%d", middle+j);
		assert(j>0);
		assert(ch != EOF);
		dest= middle[j-1];
		middle[j-1]= -1;
		move.start= no2coords(start);
		move.destination= no2coords(dest);
		for(k=0; k<j; k++)
			move.middle[k]= no2coords(middle[k]);
		poss_moves= get_all_moves(&pos);
		move= match_move(move, &poss_moves);
		assert(move.start.x > -1);
		do_move(&pos, move);
		insert_move(move);
		
		if( fscanf(f, "%d", &start) == EOF ) break;
		for(j=0; ((ch= getc(f))!=' ') && (ch != EOF); j++)
			fscanf(f, "%d", middle+j);
		assert(j>0);
		assert(ch != EOF);
		dest= middle[j-1];
		middle[j]= -1;
		move.start= no2coords(start);
		move.destination= no2coords(dest);
		for(k=0; k<j; k++)
			move.middle[k]= no2coords(middle[k]);
		poss_moves= get_all_moves(&pos);
		move= match_move(move, &poss_moves);
		assert(move.start.x > -1);
		do_move(&pos, move);
		insert_move(move);
	}
	fclose(f);
}

void load_from_history(Position *position)
{
	int i;

	initialize_board(position);
	for(i=0; i<history.count; i++)
		do_move(position, history.list[i]);
}


int coords2no(Sq_coords coords)
{
	return (coords.x/2 + 1) + coords.y*5;
}

Sq_coords no2coords(int no)
{
	Sq_coords coords;
	coords.x= ((no - 1) % 5) * 2 + (((no - 1) % 10) / 5);
	coords.y= (no - 1) / 5;
	return coords;
}
