/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
 * rules_int.h                                               *
 * part of checkers project, header file for rules_int.c     *
 * Miroslav Tynovsky 02/2004                                 *
\* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#define BOARDSIZE 10 
#define MAX_CAPTURES 16 /* horni odhad poctu moznych preskoku */
#define promote(x) ((x) * 2)
#define degrade(x) ((x) / 2)

typedef enum {
	BLACK= -1,
	WHITE= 1
} Player;

typedef enum { 
	PAWN_STEP, PAWN_JUMP, KING_STEP, KING_JUMP, 
	PROMOTION, JUMP_PROMOTION
} Move_type;

typedef enum {
	BLACK_KING= -2,
	BLACK_PAWN= -1,
	NONE= 0,
	WHITE_PAWN= 1,
	WHITE_KING= 2,
} Piece;

typedef enum {
	PROCEEDING,
	WHITE_WINS,
	BLACK_WINS,
	DRAW,
	DRAW_RP /* ... */
} Game_state;

typedef enum { NW, NE, SW, SE } Direction;

typedef struct {
	int x;
	int y;
} Sq_coords;

typedef struct {
	Sq_coords start;
	Sq_coords middle[MAX_CAPTURES - 1];
	Sq_coords destination;
	int jump_length;
	Move_type move_type;
	Sq_coords captured_coords[MAX_CAPTURES];
	Piece captured_pieces[MAX_CAPTURES];
} Move;

typedef struct {
	Move *list;
	int count;
	int allocated;
} Moves;

typedef struct {
	Piece placement[BOARDSIZE][BOARDSIZE];
	Player onmove;
} Position;

typedef struct {
	Position position;
	int frequency;
} Occured_position;

#define is_black_square(x, y) ((((x)+(y)) % 2)==0)

void initialize_board(Position *position);

Game_state get_game_state(Position *position, 
                          Occured_position *occured_positions, 
                          Move *last_move);

Moves get_all_moves(Position *position);

void do_move(Position *position, Move move);

void undo_move(Position *position, Move move);

int is_member(Sq_coords sq_start, Sq_coords sq_dest, Moves moves);

Moves select_moves_by_start(Sq_coords sq_start, Moves moves);

int eq_positions(Position position1, Position position2);

Move match_move(Move move, const Moves *moves);
